/*
scrfix 1.0 - Fix WSL directory structure to allow the current (non-root) user to run screen(1) without sudo.

To compile and set up:
* Pick a bin folder for the resulting scrfix utility; e.g., $HOME/bin or /usr/bin
* Compile with cc -o scrfix scrfix.c
* Move scrfix to the chosen folder and cd to that folder.
* As root, chown root:root scrfix
* As root, chmod 4755 scrfix

From then on, run scrfix once every time wsl starts an instance.
Then screen(1) will work without a sudo.

Caveats:
* Only tested under WSL Ubuntu distro.

Author: Doug Lee, first released Dec 16 2021.

Copyright (C) 2021-2022 Doug Lee. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.

* The names of the copyright holders and contributors may not be used to
  endorse or promote products derived from this software without specific
  prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <stdio.h>  /* for printf(3) */
#include <stdlib.h>  /* for exit(3) */
#include <unistd.h>  /* for geteuid(2) */
#include <string.h>  /* for strerror(3) */
#include <sys/stat.h>  /* for mkdir(2) and chmod(2) */
#include <errno.h>  /* Required for ld(1) to stop complaining about a TLS/non-TLS mismatch */

extern int errno;

void errExit(const char *name) {
	printf("%s: Error %d - %s\n", name, errno, strerror(errno));
	exit(-1);
}

int main(int argc, char **argv) {
	char path[] = "/run/screen";
	if (geteuid() != 0) {
		printf("Must be run as root or be stored as setuid root\n");
		exit(-1);
	}
	if (mkdir(path, 0777) < 0 && errno != 17) {
		/* 17 means already exists; that's fine. */
		errExit("mkdir");
	}
	/* That's still umasked, so we still have to chmod directly. */
	if (chmod(path, 0777) < 0) {
		errExit("mkdir");
	}
	printf("Success\n");
	exit(0);
}
